@@ -124,13 +124,27 @@ fn main() {
 
     // These constant initializers are unusable without knowledge of which type they're for; adding
     // the information here to build explicit consts
-    let struct_initializers = [
-        ("SOCK_IPV4_EP_ANY", "sock_udp_ep_t"),
-        ("SOCK_IPV6_EP_ANY", "sock_udp_ep_t"),
-        ("MUTEX_INIT", "mutex_t"),
+    let macro_functions = [
+        ("SOCK_IPV4_EP_ANY", "sock_udp_ep_t", "void", true),
+        ("SOCK_IPV6_EP_ANY", "sock_udp_ep_t", "void", true),
+        ("MUTEX_INIT", "mutex_t", "void", true),
         // neither C2Rust nor bindgen understand the cast without help
-        ("STATUS_NOT_FOUND", "thread_status_t"),
+        ("STATUS_NOT_FOUND", "thread_status_t", "void", true),
+        // If any board is ever added that works completely differently, this'll have to go behind
+        // a feature-gate
+        ("GPIO_PIN", "gpio_t", "unsigned port, unsigned pin", true),
     ];
+    let mut macro_functions: Vec<_> = macro_functions
+        .iter()
+        .map(|(macro_name, return_type, args, is_const)| {
+            (macro_name.to_string(), *return_type, *args, *is_const)
+        })
+        .collect();
+    for i in 0..8 {
+        macro_functions.push((format!("LED{}_ON", i), "void", "void", false));
+        macro_functions.push((format!("LED{}_OFF", i), "void", "void", false));
+        macro_functions.push((format!("LED{}_TOGGLE", i), "void", "void", false));
+    }
 
     let mut c_code = String::new();
     std::fs::File::open("riot-c2rust.h")
@@ -138,28 +152,47 @@ fn main() {
         .read_to_string(&mut c_code)
         .expect("Failed to read riot-c2rust.h");
 
-    for (macro_name, type_name) in struct_initializers.iter() {
+    for (macro_name, return_type, args, _is_const) in macro_functions.iter() {
         // The ifdef guards make errors easier to spot: A "cannot find function
-        // `init_SOCK_IPV6_EP_ANY` in crate `riot_sys`" can lead one to check whether
+        // `macro_SOCK_IPV6_EP_ANY` in crate `riot_sys`" can lead one to check whether
         // SOCK_IPV6_EP_ANY is really defined, whereas if the macro is missing, C2Rust would
         // produce a run-time panic, and the compiler would reject that in a const function.
         //
         // This is more reliable than the previous approach of trying to defined a `-DSOME_MODULE`
         // condition, also because there may not even be a module that gives a precise condition.
-        write!(
-            c_code,
-            r"
+        if *return_type == "void" {
+            // in C, assigning and returning void is special
+            write!(
+                c_code,
+                r"
 
 #ifdef {macro_name}
-{type_name} init_{macro_name}(void) {{
-    {type_name} result = {macro_name};
+{return_type} macro_{macro_name}({args}) {{
+    {macro_name};
+}}
+#endif
+                ",
+                return_type = return_type,
+                macro_name = macro_name,
+                args = args,
+            )
+        } else {
+            write!(
+                c_code,
+                r"
+
+#ifdef {macro_name}
+{return_type} macro_{macro_name}({args}) {{
+    {return_type} result = {macro_name};
     return result;
 }}
 #endif
-            ",
-            type_name = type_name,
-            macro_name = macro_name,
-        )
+                ",
+                return_type = return_type,
+                macro_name = macro_name,
+                args = args,
+            )
+        }
         .unwrap();
     }
 
@@ -311,27 +344,37 @@ fn main() {
 
     for chunk in functionchunks {
         let funcname = &chunk[..chunk.find('(').expect("Function has parentheses somewhere")];
-        let new_prefix = match funcname {
+        let macro_details = if funcname.len() > 5 && &funcname[..6] == "macro_" {
+            macro_functions
+                .iter()
+                .filter(|(macro_name, _, _, _)| &funcname[6..] == *macro_name)
+                .next()
+        } else {
+            None
+        };
+        let new_prefix = match (funcname, macro_details) {
             // used as a callback, therefore does need the extern "C" -- FIXME probably worth a RIOT issue
-            "_evtimer_msg_handler" | "_evtimer_mbox_handler" => function_original_prefix,
+            ("_evtimer_msg_handler" | "_evtimer_mbox_handler", _) => function_original_prefix,
 
             // Assigned by CMSIS to some const; see also riot-c2rust.h
-            "__masked_builtin_arm_get_fpscr" | "__masked_builtin_arm_set_fpscr" => {
+            ("__masked_builtin_arm_get_fpscr" | "__masked_builtin_arm_set_fpscr", _) => {
                 function_original_prefix
             }
 
             // same problem but from C2Rust's --translate-const-macros
-            "__NVIC_SetPriority" => function_original_prefix,
+            ("__NVIC_SetPriority", _) => function_original_prefix,
 
-            // As below (no need for extern), and they are const by construction.
-            s if s.len() > 5
-                && &s[..5] == "init_"
-                && struct_initializers
-                    .iter()
-                    .any(|(macro_name, _)| &s[5..] == *macro_name) =>
-            {
+            // As below (no need for extern), and they are const as declared ni the macro_functions
+            // list.
+            (_, Some((_, _, _, is_const))) => {
                 // No "pub" because that's already a "pub" in front of it, they were never static
-                "const fn "
+                match is_const {
+                    // FIXME: These should be unsafe -- just because most of them are const doesn't
+                    // necessrily mean they're safe (just the first few happened to be, but that's
+                    // not this crate's place to assert)
+                    true => "const unsafe fn ",
+                    false => "unsafe fn ",
+                }
             }
 
             // The rest we don't need to call through the extern convention, but let's please make
@@ -349,4 +392,57 @@ fn main() {
         .expect("Failed to create riot_c2rust_replaced.rs")
         .write(rustcode.as_bytes())
         .expect("Failed to write to riot_c2rust_replaced.rs");
+
+    // Pub uses of inline right into the main lib.rs
+    //
+    // This is primarily for things that can really come from either backend (eg. irq functions
+    // that are regular on native but static inline on others), and for convenience stuff like
+    // macro_.
+    //
+    // Some functions are also in because they're innocuous enough.
+    let mut toplevel_from_inline: Vec<String> = [
+        "bluetil_ad_add_flags",
+        "coap_get_code_raw",
+        "coap_get_total_hdr_len",
+        "gnrc_netapi_dispatch_send",
+        "gnrc_netif_ipv6_addrs_get",
+        "gnrc_netreg_entry_init_pid",
+        "gpio_is_valid",
+        "irq_disable",
+        "irq_is_enabled",
+        "irq_is_in",
+        "irq_restore",
+        "mutex_trylock",
+        "pid_is_valid",
+        "shell_run_forever",
+        "sock_udp_recv",
+        "sock_udp_send",
+        "thread_get",
+        "thread_getpid",
+        "thread_get_unchecked",
+        "ztimer_spin",
+    ]
+    .iter()
+    .map(|name| name.to_string())
+    .collect();
+    for (macro_name, _, _, _) in macro_functions.iter() {
+        toplevel_from_inline.push(format!("macro_{}", macro_name));
+    }
+    let toplevel_from_inline: Vec<String> = toplevel_from_inline
+        .drain(..)
+        .filter(|s: &String| rustcode.contains(s))
+        .collect();
+    let toplevel_from_inline_filename = out_path.join("toplevel_from_inline.rs");
+    std::fs::File::create(toplevel_from_inline_filename)
+        .expect("Failed to create toplevel_from_inline.rs")
+        .write(
+            format!(
+                "
+               pub use inline::{{ {} }};
+           ",
+                toplevel_from_inline.join(",\n")
+            )
+            .as_bytes(),
+        )
+        .expect("Failed to write to toplevel_from_inline.rs");
 }
