@@ -75,18 +75,27 @@
 #include <periph/wdt.h>
 #endif
 
+#ifdef MODULE_CORE_THREAD_FLAGS
+#include <thread_flags.h>
+#endif
 #include <net/gnrc.h>
 #include <net/gnrc/udp.h>
 #include <net/gnrc/pktbuf.h>
 #include <net/gnrc/ipv6.h>
 #include <net/gnrc/nettype.h>
 #include <net/gnrc/netapi.h>
+#ifdef MODULE_GNRC_ICMPV6
+#include "net/gnrc/icmpv6.h"
+#endif
 #ifdef MODULE_SOCK
 #include <net/sock.h>
 #endif
 #ifdef MODULE_SOCK_UDP
 #include <net/sock/udp.h>
 #endif
+#ifdef MODULE_SOCK_ASYNC
+#include <net/sock/async.h>
+#endif
 #ifdef MODULE_GCOAP
 #include <net/gcoap.h>
 #endif
@@ -103,6 +112,9 @@
 #ifdef MODULE_ZTIMER
 #include <ztimer.h>
 #endif
+#ifdef MODULE_ZTIMER_PERIODIC
+#include <ztimer/periodic.h>
+#endif
 #include <mutex.h>
 #ifdef MODULE_CORD_COMMON
 #include <net/cord/common.h>
@@ -152,3 +164,8 @@
 #ifdef MODULE_MICROBIT
 #include "microbit.h"
 #endif
+
+// Note that while the actual definitions are always in board.h, this defines
+// the fallback macros that make sure that in the LED macros' absence,
+// fallbacks are there.
+#include <led.h>
